/*
 * Program showing a graph that displays values received from a serial connection
 * The data received should be comma separated strings representing integer values
 * In this program there are 4 parameters, adapt if needed
 * For information on the used graph library, check the library
 * reference of the grafica library at the procesing website
 *
 * Author P.Peters
 * May 2015
 * Version 0.5
 */
 
// use serial communication
import processing.serial.*;
// and grafica library
import grafica.*;

// character indicating end-of-line
byte linefeed = 10;

// data array to store the values received from the serial connection after conversion
// length depends on number of parameters received over the serial connection
int[] data = { 0, 0, 0, 0 };

// Graph to display
GPlot plot0, plot1, plot2, plot3;
GPointsArray points0, points1, points2, points3;

//  number of plot points
int nPoints = 100;
int pointsNbr = 0;

// and the serial connection
Serial myPort;

void setup() {
  // canvas size 
  size(500, 350);
  // Prepare the points for the plot
  points0 = new GPointsArray(nPoints);
  points1 = new GPointsArray(nPoints);
  points2 = new GPointsArray(nPoints);
  points3 = new GPointsArray(nPoints);
  // initialise all points at 0
  for (int i = 0; i < nPoints; i++) {
    points0.add(i, 0);
    points1.add(i, 0);
    points2.add(i, 0);
    points3.add(i, 0);
  }

  // Create new plots and set the positions on the screen
  plot0 = new GPlot(this, 25, 25);
  plot1 = new GPlot(this, 25, 25);
  plot2 = new GPlot(this, 25, 25);
  plot3 = new GPlot(this, 25, 25);

  // Set the plot title, axis labels, limits and point color/size
  plot0.setTitleText("A timed plot");
  plot0.getXAxis().setAxisLabelText("x axis");
  plot0.getYAxis().setAxisLabelText("y axis");
  plot0.setXLim(1, 100);
  plot0.setYLim(0, 1024);
  plot0.setPointColor(color(255, 0, 0));
  plot0.setPointSize(2);
  // Link the points to the plot
  plot0.setPoints(points0);
  // Set the plot limits and point color/size
  plot1.setXLim(1, 100);
  plot1.setYLim(0, 1024);
  plot1.setPointColor(color(0, 255, 0));
  plot1.setPointSize(2);
  // Link the points to the plot
  plot1.setPoints(points1);
  // Set the plot limits and point color/size
  plot2.setXLim(1, 100);
  plot2.setYLim(0, 1024);
  plot2.setPointColor(color(0, 0, 255));
  plot2.setPointSize(2);
  // Link the points to the plot
  plot2.setPoints(points2);
  // Set the plot limits and point color/size
  plot3.setXLim(1, 100);
  plot3.setYLim(0, 1024);
  plot3.setPointColor(color(0, 0, 0));
  plot3.setPointSize(2);
  // Link the points to the plot
  plot3.setPoints(points3);

  // display list of available serial ports
  for (int i=0; i<Serial.list().length; i++) println("index "+i+" : "+Serial.list()[i]);

  // the index used in Serial.list()[index] should be determined
  // when it is known which serial connection to use, this differs
  // per computer depending on how many serial ports you have...
  // choose from what is displayed at first run, it should match the port used
  // in the Arduino IDE (if you are using an Arduino)
  int index = 0;
  // create serial connection
  myPort = new Serial(this, Serial.list()[index], 115200);

  // this is used to notify the other side (the Arduino) that we're ready
  // I use it to start sending out the sensor parameters repetitively
  myPort.write("r\n");
}

void draw() {
  if (checkForNewData()) {
    // new data available, so add new points to plot
    newPoint(plot0, data[0]);
    newPoint(plot1, data[1]);
    newPoint(plot2, data[2]);
    newPoint(plot3, data[3]);
    
    // redraw plots
    background(150);
    plot0.beginDraw();
    plot0.drawBackground();
    plot0.drawBox();
    plot0.drawXAxis();
    plot0.drawYAxis();
    plot0.drawTitle();
    plot0.drawPoints();
    plot0.endDraw();
    
    plot1.beginDraw();
    plot1.drawPoints();
    plot1.endDraw();
    
    plot2.beginDraw();
    plot2.drawPoints();
    plot2.endDraw();
    
    plot3.beginDraw();
    plot3.drawPoints();
    plot3.endDraw();
  }
}

// add a new point to the end of the plot
void newPoint(GPlot p, int val) {
  GPointsArray pts = p.getPoints();
  // move all points left
  for (int i=1; i<nPoints; i++) pts.setY(i-1, pts.getY(i));
  // set last point to new value
  pts.setY(nPoints-1, val);
  // refresh plot
  p.setPoints(pts);
}

//  check if new data is available and translate it to the data array
boolean checkForNewData() {
  // if data available
  if (myPort.available() > 0) { 
    // you know you received a string with comma separated paramaters, so read it
    String inString = myPort.readStringUntil(linefeed);
    // if it is not empty
    if (inString != null) {
      // dislay it (just to check)
      print(inString);
      // and split the string into an array of ints
      // first cleanup the received string (remove leading and trailing spaces and newline)
      // then split into array of strings (using the comma separator)
      // then convert into array of ints
      data = int(split(trim(inString), ',')); 
      // create a new maximum if needed
      //for (int i=0; i < data.length; i++) {
      //  dataMax = max(dataMax, data[i]);
      //}
      return true;
    }
  }
  return false;
}

// this is only useful if you actually also check for data reception in the Arduino
// and do something with that data...
void keyPressed() {
  // send the key pressed
  myPort.write(key);
  // and a newline
  myPort.write('\n');
}